unit _Main;

interface
uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, ExtCtrls, Utils, Menus, ComCtrls, ExtDlgs, ToolWin;

const
	CardText: array[0..7] of string = ('7', '8', '9', '10', 'spodek', 'svrek', 'krl', 'eso');
	GameText: array[0..6] of string = ('hra', 'betl', 'durch s kartou', 'durch s chytrm',
		'durch', 'loen betl', 'loen durch');
	FlekText: array[1..4] of string = ('Flek', 'Re', 'Tutti', 'Boty');

type
	TCardColor = 0..3;
	TCard = 0..31;
	TPlayers = 0..3;

	TStat = record
		Prosla: boolean;
		VzalTrumf: boolean;
		VzalTrumfem: TPlayers;
		Eso: boolean;
		Zbyva: byte;
	end;

	TCards = class
		Card: array[1..32] of TCard;
		Count: byte;
		Tag: array[1..32] of integer;
		Sorted: boolean;
		procedure Add(ACard: TCard);
		procedure Remove(ACard: TCard);
		procedure Sort;
		function IsThere(ACard: TCard): boolean;
		function WhereIs(ACard: TCard): byte;
		function ColorCount(AColor: TCardColor): byte;
		function BestCards(AColor: TCardColor): byte;
	end;

	TTrick = class(TCards)
		Best: TCard;
		Owner: TPlayers;
		Color: TCardColor;
		Trump: boolean;
	end;

	TPlayer = class
	public
		Me: TPlayers;
		Hand: TCards;
		Enemies: array[TPlayers] of Shortint;
		Points: byte;
		Calls: byte;
		TrumpCall: boolean;
		Tricks: TCards;
		Money: integer;
		HraMoney: integer;
		SedmaMoney: integer;
		MeWith: boolean;
		Samec: boolean;
		constructor Create;
		procedure ChooseTrump; virtual;
		function ChooseColor: byte; virtual;
		function ChooseFlek: boolean; virtual;
		procedure Play; virtual;
		procedure CutCards; virtual;
		procedure ShowCards;
		procedure ShowTrickCard(TrickCard: byte);
		procedure Say(S: string);
		procedure SayGetTrick;
		function PossibleCards(Possible: TCards): byte;
		function PossiblePlay(ACard: TCard): boolean;
	end;

	THumanPlayer = class(TPlayer)
	public
		procedure ChooseTrump; override;
		procedure ChooseTrumpMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
		procedure ChooseFolkTrumpMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
		function ChooseColor: byte; override;
		function ChooseFlek: boolean; override;
		procedure CutCards; override;
		procedure Play; override;
		procedure PlayClick(Sender: TObject);
	end;

	TPoints = class
		Points: byte;
		Calls: byte;
		TrumpCall: boolean;
		procedure Add(APlayer: TPlayers);
		function Hundred: boolean;
		function BestCall: byte;
		function Sum: byte;
	end;

	TCardImage = class(TImage)
	public
		IsFront: boolean;
		procedure Front;
		procedure Back;
	end;

	TMain = class(TForm)
		StatusBar: TStatusBar;
		PlayWithMenu: TPopupMenu;
		PlayWith7: TMenuItem;
		PlayWith3: TMenuItem;
		PlayWith6: TMenuItem;
		PlayWith5: TMenuItem;
		PlayWith4: TMenuItem;
		PlayWith2: TMenuItem;
		PlayWith1: TMenuItem;
		PlayWith0: TMenuItem;
		MainMenu: TMainMenu;
		GameMenu: TMenuItem;
		GameNew: TMenuItem;
		GameBreak2: TMenuItem;
		GameExit: TMenuItem;
		GameOptions: TMenuItem;
		GameBreak1: TMenuItem;
		HelpMenu: TMenuItem;
		HelpAbout: TMenuItem;
		HelpBreak1: TMenuItem;
		HelpTopics: TMenuItem;
		GamePanel: TPanel;
		ColorImage: TImage;
		GameLabel: TLabel;
		WithLabel: TLabel;
		HLine2: TBevel;
		HLine1: TBevel;
		VLine1: TBevel;
		VLine4: TBevel;
		VLine2: TBevel;
		VLine3: TBevel;
		GameMoney: TMenuItem;
		HelpTips: TMenuItem;
		TrickTimer: TTimer;
		SayPanel: TPanel;
		SayLabel: TLabel;
    ShowTimer: TTimer;
		procedure GameExitClick(Sender: TObject);
		procedure GameNewClick(Sender: TObject);
		procedure PlayWithClick(Sender: TObject);
		procedure FormResize(Sender: TObject);
		procedure FormHint(Sender: TObject);
		procedure GameOptionsClick(Sender: TObject);
		procedure HelpAboutClick(Sender: TObject);
		procedure HelpTopicsClick(Sender: TObject);
		procedure FormKeyPress(Sender: TObject; var Key: Char);
		procedure GameMoneyClick(Sender: TObject);
		procedure HelpTipsClick(Sender: TObject);
		procedure FormActivate(Sender: TObject);
		procedure GetTrick(Sender: TObject);
		procedure GetTrickClick(Sender: TObject);
		procedure GetTrickKeyPress(Sender: TObject; var Key: Char);
		procedure GetTrickReset;
		procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ShowTimerTimer(Sender: TObject);
	public
		function BetterRank(Better, Worse: TCard): boolean;
		function BetterCard(Better, Worse: TCard): boolean;
		procedure MoveControl(Control: TControl; X, Y: integer);
		procedure MoveCard(ACard: TCard; X, Y: integer);
		procedure MixCards;
		procedure NewGame;
		procedure PlayCard(ACard: TCard);
		procedure DealCards;
		procedure CutIt(Position: TCard);
		procedure PrepareGetTrick;
		procedure EndGame;
		procedure EndNoFlek;
		procedure ShowResults;
		function ShowModalCenter(AForm: TForm): integer;
		procedure ShowGameInfo(APlayer: TPlayers);
		procedure ShowLines;
	end;

var
	Main: TMain;
	Player: array[TPlayers] of TPlayer;
	Trick: TTrick;
	Playing: TPlayers;
	Packet: TCards;
	Trump: TCardColor;
	Trumper: TPlayers;
	GameType: byte;
	TrumpCard: TCard;
	TrumpWith: TCard;
	EndOfGame: boolean;
	CardImage: array[TCard] of TCardImage;
	CardFront: array[TCard] of TBitmap;
	CardBack: TBitmap;
	ColorIcon: array[TCardColor] of TIcon;
	CardSpace: TPoint;
	CardSize: TPoint;
	Stat: array[TCardColor] of TStat;
	Flek: byte;
	Sedma: shortint;
	Kilo: shortint;
	ResultColor: TColor;

implementation
{$R *.DFM}

uses _Results, _SetColor, _SetFlek, _Options, _SayIt, _About, _Moneys, _Tips,
	_SetCut;

{===========================================================================================}
{===========================================================================================}
{===========================================================================================}

procedure TMain.FormActivate(Sender: TObject);
var i: byte;
begin
	Randomize;
	EndOfGame:= True;
	Application.OnHint:= FormHint;
	CardBack:= TBitmap.Create;
	CardBack.LoadFromFile(ExtractFilePath(Application.ExeName) + 'Cards\card00.bmp');
	{CardBack.LoadFromResourceName(0, 'CARD00');}
	for i:= 0 to 31 do begin
		CardImage[i]:= TCardImage.Create(Main);
		CardFront[i]:= TBitmap.Create;
		CardFront[i].LoadFromFile(ExtractFilePath(Application.ExeName) + Format('Cards\card%.2d.bmp', [i+1]));
		{CardFront[i].LoadFromResourceName(0, Format('CARD%.2d', [i+1]));}
		with CardImage[i] do begin
			Parent:= Main;
			Tag:= i;
			AutoSize:= True;
			Hide;
		end;
	end;
	for i:= 0 to 3 do begin
		ColorIcon[i]:= TIcon.Create;
		ColorIcon[i].LoadFromFile(Format('%sCards\color%d.ico', [ExtractFilePath(Application.ExeName), i]));
	end;
	CardSize.X:= CardBack.Width;
	CardSize.Y:= CardBack.Height;
	Packet:= TCards.Create;
	Trick:= TTrick.Create;
	Player[0]:= THumanPlayer.Create;
	for i:= 1 to 3 do Player[i]:= TPlayer.Create;
	for i:= 0 to 3 do Player[i].Me:= i;
	Moneys.Load;
	Options.Load;
	Options.Apply;
	if Tips.ShowTips.Checked then ShowModalCenter(Tips);
	if Options.StartNewGame.Checked then NewGame;
end;

procedure TMain.NewGame;
var i, j: byte;
begin
	GetTrickReset;
	ColorImage.Hide;
	Main.Hint:= '';
	GamePanel.Hide;
	EndOfGame:= False;
	Trick.Count:= 0;
	for i:= 0 to 3 do with Player[i] do begin
		Points:= 0;
		Calls:= 0;
		TrumpCall:= False;
		for j:= 0 to 3 do Enemies[j]:= 0;
	end;
	for i:= 0 to 31 do begin
		CardImage[i].Hide;
		{CardImage[i].BringToFront;}
		CardImage[i].OnMouseDown:= nil;
	end;
	for i:= 0 to 3 do with Stat[i] do begin
		Prosla:= False;
		VzalTrumf:= False;
		VzalTrumfem:= 0;
		Eso:= False;
		Zbyva:= 8;
	end;
	Flek:= 0;
	Sedma:= 0;
	Kilo:= 0;

	if Options.Mixing.Checked then MixCards;
	Player[Trumper].CutCards;
	Trumper:= (Trumper + 1) mod 4;
	ShowGameInfo(Trumper);
	for i:= 0 to 3 do Player[i].Hand.Count:= 0;
	for i:= 0 to 3 do for j:= 1 to 4 do Player[(i + Trumper) mod 4].Hand.Add(Packet.Card[i*4 + j]);
	if Trumper=0 then Player[0].Hand.Sort;
	for i:= 4 to 7 do for j:= 1 to 4 do Player[(i + Trumper) mod 4].Hand.Add(Packet.Card[i*4 + j]);
	Player[Trumper].ChooseTrump;
end;

procedure TPlayer.ChooseTrump;
var i, j: byte;
	TrumpColor: array[TCardColor] of TCards;
	ColorPower: array[TCardColor] of Word;
function RemoveSelect(ACard: TCard): boolean;
	begin
		Result:= False;
		TrumpColor[Trump].Remove(8*Trump + ACard);
		if TrumpColor[Trump].Count = 0 then TrumpCard:= 8*Trump + ACard
		else Result:= True;
	end;
function IsThereSelect(ACard: TCard): boolean;
	begin
		Result:= TrumpColor[Trump].IsThere(8*Trump + ACard);
		if not Result then TrumpWith:= 8*Trump + ACard;
	end;
begin
	for i:= 0 to 3 do TrumpColor[i]:= TCards.Create;

	{=========================== strategie pro vbr trumf ===========================}

	{rozazen karet do barev}
	for i:= 1 to 4 do TrumpColor[Hand.Card[i] div 8].Add(Hand.Card[i]);
	{uren sly barvy}
	for i:= 0 to 3 do begin
		ColorPower[i]:= 0;
		for j:= 1 to TrumpColor[i].Count do case TrumpColor[i].Card[j] mod 8 of
			0: Inc(ColorPower[i], 10);
			3: Inc(ColorPower[i], 25);
			5: if TrumpColor[i].IsThere(8*i + 6) then Inc(ColorPower[i], 30)
				else Inc(ColorPower[i], 10);
			6: Inc(ColorPower[i], 10);
			7: Inc(ColorPower[i], 30);
			else Inc(ColorPower[i], TrumpColor[i].Card[j] mod 8);
		end;
	end;

	{vbr trumfu}
	Trump:= 0;
	for i:= 1 to 3 do if TrumpColor[i].Count > TrumpColor[Trump].Count then Trump:= i
	else if TrumpColor[i].Count = TrumpColor[Trump].Count then
		if ColorPower[i] > ColorPower[Trump] then Trump:= i;

	{strategie vbru spolukarty}
	if IsThereSelect(7) then if IsThereSelect(3) then {nehraju s kartami, kter drm}
		if TrumpColor[Trump].IsThere(8*Trump + 6) or TrumpColor[Trump].IsThere(8*Trump + 5) then
			{kdy mm eso, destku, (krle nebo svrka) a spodka, hraju samce}
			if TrumpColor[Trump].IsThere(8*Trump + 4) then TrumpWith:= 8*Trump + 7
			{kdy si trhm hlku, radi hraju se spodkem}
			else TrumpWith:= 8*Trump + 4
		else TrumpWith:= 8*Trump + 6; {jinak s krlem}

	{strategie odloen karty}
	TrumpColor[Trump].Sort;
	if RemoveSelect(7) then if RemoveSelect(3) then if RemoveSelect(0) then
		{neukazuji soupem eso, destku a sedmiku, potom od nejni}
		TrumpCard:= TrumpColor[Trump].Card[1];

	for i:= 0 to 3 do TrumpColor[i].Free;
	Main.DealCards;
end;

procedure THumanPlayer.ChooseTrump;
var i, j: byte;
begin
	if not Options.Trumping.Checked then inherited ChooseTrump
	else begin
		Main.Hint:= 'Vol trumfy';
		for i:= 1 to 4 do CardImage[Hand.Card[i]].Front;
		for i:= 5 to 8 do CardImage[Hand.Card[i]].Back;
		for i:= 5 to 8 do CardImage[Hand.Card[i]].BringToFront;
		for i:= 1 to 4 do CardImage[Hand.Card[i]].BringToFront;
		for i:= 1 to 3 do for j:= 1 to 8 do CardImage[Player[i].Hand.Card[j]].Back;
		for i:= 1 to 3 do for j:= 1 to 8 do CardImage[Player[i].Hand.Card[j]].BringToFront;
		for i:= 0 to 3 do Player[i].ShowCards;
		for i:= 1 to 4 do CardImage[Hand.Card[i]].OnMouseDown:= ChooseTrumpMouseDown;
		for i:= 5 to 8 do CardImage[Hand.Card[i]].OnMouseDown:= ChooseFolkTrumpMouseDown;
	end;
end;

procedure THumanPlayer.ChooseTrumpMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	TrumpCard:= TCardImage(Sender).Tag;
	Trump:= TrumpCard div 8;
	if Button = mbRight then Main.PlayWithClick(Main.PlayWith7)
	else begin
		Application.Hint:= 'S jakou kartou bude hrt?';
		with Main do PlayWithMenu.Popup(
			Left + Width - ClientWidth + TCardImage(Sender).Left + X,
			Top + Height - ClientHeight + TCardImage(Sender).Top + Y);
	end;
end;

procedure THumanPlayer.ChooseFolkTrumpMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	TrumpCard:= TCardImage(Sender).Tag;
	Trump:= TrumpCard div 8;
	Main.PlayWithClick(Main.PlayWith7);
end;

procedure TMain.PlayWithClick(Sender: TObject);
var i: byte;
begin
	Main.Hint:= '';
	for i:= 1 to 8 do CardImage[Player[0].Hand.Card[i]].OnMouseDown:= nil;
	TrumpWith:= 8*Trump + TMenuItem(Sender).Tag;
	DealCards;
end;

procedure TMain.DealCards;
var i, j, k: byte;
	Fleknul: boolean;
begin
	for i:= 0 to 3 do Player[i].MeWith:= Player[i].Hand.IsThere(TrumpWith) or (i=Trumper);
	for i:= 0 to 3 do Player[i].Samec:= False;
	if Player[Trumper].Hand.IsThere(TrumpWith) then Player[Trumper].Samec:= True;
	for i:= 0 to 3 do Player[i].Hand.Sort;
	for i:= 1 to 8 do CardImage[Player[0].Hand.Card[i]].Front;
	if Options.OpenGame.Checked then
		for i:= 1 to 3 do for j:= 1 to 8 do CardImage[Player[i].Hand.Card[j]].Front
	else for i:= 1 to 3 do for j:= 1 to 8 do CardImage[Player[i].Hand.Card[j]].Back;
	for i:= 0 to 31 do CardImage[i].BringToFront;
	for i:= 0 to 3 do Player[i].ShowCards;
	GameType:= 0;
	GamePanel.Show;
	GameLabel.Caption:= Capitalize(GameText[GameType]);
	WithLabel.Caption:= Capitalize(CardText[TrumpWith mod 8]);
	Playing:= Trumper;
	if Trumper=0 then MoveCard(TrumpCard, CardImage[TrumpCard].Left, CardImage[TrumpCard].Top - Min(CardSpace.Y div 5, 10));
	if Trumper=0 then Refresh;

	for i:= 1 to 6 do TMenuItem(SetColor.FindComponent(Format('GameType%d', [i]))).Enabled:= True;
	{potvrzen barvy}
	i:= Trumper; j:= 0;
	while j < 4 do begin
		k:= Player[i mod 4].ChooseColor;
		if k <= GameType then Inc(j)
		else begin {zvolil vy patnou barvu}
			GameLabel.Caption:= Capitalize(GameText[GameType]);
			if GameType = 2 then begin
				ColorImage.Picture.Icon.Assign(ColorIcon[Trump]);
				ColorImage.Show;
				WithLabel.Caption:= Capitalize(CardText[TrumpWith mod 8]);
			end
			else begin
				WithLabel.Caption:= '';
				ColorImage.Hide;
			end;
			Playing:= i mod 4;
			GameType:= k;
			j:= 1;
		end;
		Inc(i);
	end;

	if Trumper <> 0 then begin
		if not Options.OpenGame.Checked then CardImage[TrumpCard].Front;
		if not Options.OpenGame.Checked then Player[Trumper].ShowCards;
		if not Options.OpenGame.Checked then CardImage[TrumpCard].SendToBack;
		MoveCard(TrumpCard, CardImage[TrumpCard].Left, CardImage[TrumpCard].Top - Min(CardSpace.Y div 5, 10));
	end;
	if GameType=0 then ColorImage.Picture.Assign(ColorIcon[Trump]);
	ColorImage.Show;

	{Fleky}
	SetFlek.Flek.Enabled:= True;
	SetFlek.Neflek.Enabled:= True;
	SetFlek.Kilo.Enabled:= True;
	SetFlek.Sedma.Enabled:= True;
	if GameType = 0 then i:= Playing else i:= Playing + 1;
	j:= i - Playing;
	Fleknul:= False;
	while j < 4 do begin
		if not Player[i mod 4].ChooseFlek then Inc(j)
		else begin {fleknul}
			Fleknul:= True;
			j:= 1;
			for k:= 0 to 3 do Player[k].Enemies[i mod 4]:= -Ord11(Player[i mod 4].MeWith xor Player[k].MeWith);
		end;
		Inc(i);
	end;

	if not Options.OpenGame.Checked and (Trumper <> 0) then begin
		CardImage[TrumpCard].Back;
		for i:= 1 to 8 do CardImage[Player[Trumper].Hand.Card[i]].BringToFront;
	end
	else MoveCard(TrumpCard, CardImage[TrumpCard].Left, CardImage[TrumpCard].Top + Min(CardSpace.Y div 5, 10));

	if (GameType <> 0) or Fleknul then Player[Playing].Play
	else EndNoFlek;
end;

function TPlayer.ChooseColor: byte;
begin
	{strategie pro zvolen patn barvy}
	Result:= 0;
end;

function THumanPlayer.ChooseColor: byte;
var i: byte;
begin
	if not Options.Coloring.Checked then Result:= inherited ChooseColor
	else begin
		for i:= 1 to GameType do TMenuItem(SetColor.FindComponent(Format('GameType%d', [i]))).Enabled:= False;
		SetColor.CurrentGameTypeLabel.Caption:= Format('Hraje se %s.', [GameText[GameType]]);
		i:= Main.ShowModalCenter(SetColor);
		if i = 0 then Result:= 0
		else Result:= i-1;
	end;
end;

function TPlayer.ChooseFlek: boolean;
var i: byte;
begin
	Result:= False;
	{uren neptel}
	if MeWith then {hraju s}
		if Me = Trumper then Enemies[Me]:= 1
		else for i:= 0 to 3 do if (i = Trumper) or (i = Me) then Enemies[i]:= 1 else Enemies[i]:= -1
	else begin {hraju proti}
		Enemies[Trumper]:= -1;
		Enemies[Me]:= 1;
	end;

	if MeWith then begin {hraju s}
		if (Hand.IsThere(8*Trump + 5) and Hand.IsThere(8*Trump + 6)) {mm hlku}
		and (Kilo = 0) and ((Me <> Trumper) or not Hand.IsThere(TrumpWith)) then begin {hlen povinnho kila}
			Kilo:= 1;
			if Me=Trumper then Say('Kilo') else Say('Kilo s');
			Result:= True;
		end;
	end
	else begin {hraju proti}
		if (Hand.IsThere(8*Trump + 5) or Hand.IsThere(8*Trump + 6)) and (Flek = 0) and (Kilo=0) then begin {trhm hlku}
			Flek:= 1;
			Say('Flek!');
			Result:= True;
		end;
	end;

	{flekovac strategie}
	{sedma}
	if (Hand.ColorCount(Trump) >= 3 + Ord(Options.Risk.Checked)) then
		if Sedma = 0 then {sedmu jet nikdo nehlsil}
			if (Hand.IsThere(8*Trump)) and not MeWith then begin
				Sedma:= Enemies[Trumper];
				if Sign(Sedma) = -1 then Say('7 proti') else Say('7 s');
				Result:= True;
			end
			else
		else if Sign(Sedma) = -Enemies[Trumper] then {sedmu nahlsil soupe}
			if (Abs(Sedma) = 1) and Options.Risk.Checked then begin
				Sedma:= Sedma + Sign(Sedma);
				Say('Flek na sedmu');
				Result:= True;
			end
			else
		else if Sign(Sedma) = Enemies[Trumper] then {sedmu jsme hlsili my}
			if (Abs(Sedma) = 2) and (Hand.ColorCount(Trump) >= 5 - Ord(Options.Risk.Checked)) then begin
				Sedma:= Sedma + Sign(Sedma);
				Say('Re na sedmu');
				Result:= True;
			end
			else
end;

function THumanPlayer.ChooseFlek: boolean;
var
	Fleknul: boolean;
	PovinneKilo: boolean;
function FlekujText(Flekuj: shortint; S: string): string;
	begin
		if Flekuj <> 0 then Flekuj:= Flekuj - Sign(Flekuj) * Ord((Flekuj > 0) xor (MeWith xor Odd(Flekuj)));
		if Flekuj = 0 then Result:= Format('&%s ', [Capitalize(S)])
		else Result:= Format('%s na &%s ', [FlekText[Abs(Flekuj)], S]);
		if (Flekuj <> 0) or (Me <> Trumper) then
			if (Sign(Flekuj) > 0) or ((Flekuj = 0) and MeWith) then Result:= Result + 's'
			else Result:= Result + 'proti';
	end;
function Povol(Povoluj: shortint): boolean;
	begin
		Result:= (Povoluj = 0)
		or ((Sign(Povoluj) = Enemies[Trumper]) and not Odd(Povoluj))
		or ((Sign(Povoluj) = -Enemies[Trumper]) and Odd(Povoluj));
	end;
procedure ZvysFlek(var Zvysuj: shortint);
	begin
		if Zvysuj = 0 then Zvysuj:= Enemies[Trumper]
		else Zvysuj:= Zvysuj + Sign(Zvysuj);
	end;
begin
	if not Options.Fleking.Checked then Result:= inherited ChooseFlek
	else begin
		MeWith:= Hand.IsThere(TrumpWith) or (Me=Trumper);
		Enemies[Trumper]:= Ord11(MeWith);
		Enemies[Me]:= 1;

		SetFlek.Flek.Caption:= Format('&%s!', [FlekText[Flek + Ord(not Odd(Flek) or MeWith)]]);;
		SetFlek.Sedma.Caption:= FlekujText(Sedma, '7');
		SetFlek.Kilo.Caption:= FlekujText(Kilo, 'kilo');

		SetFlek.Flek.Enabled:= not (Odd(Flek) xor MeWith);
		SetFlek.Sedma.Enabled:= Povol(Sedma);
		SetFlek.Kilo.Enabled:= Povol(Kilo);

		PovinneKilo:= False;
		if MeWith then begin {hraju s}
			if Hand.IsThere(8*Trump + 5) and Hand.IsThere(8*Trump + 6) and (Kilo = 0) then begin {mm hlku}
				SetFlek.Kilo.Enabled:= False;
				SetFlek.Kilo.Checked:= True;
				PovinneKilo:= True;
			end;
		end
		else begin {hraju proti}
			if Hand.IsThere(8*Trump + 5) or Hand.IsThere(8*Trump + 6) {trhm hlku}
			and (Flek = 0) and (Me <> Trumper) then SetFlek.Neflek.Enabled:= False;
		end;
		if Kilo <> 0 then SetFlek.Neflek.Enabled:= False;

		Fleknul:= (Main.ShowModalCenter(SetFlek) = 1) or not SetFlek.Neflek.Enabled;
		if Fleknul then Inc(Flek);
		if SetFlek.Sedma.Checked and SetFlek.Sedma.Enabled then ZvysFlek(Sedma);
		if (SetFlek.Kilo.Checked and SetFlek.Kilo.Enabled) or PovinneKilo then ZvysFlek(Kilo);

		Result:= Fleknul or (SetFlek.Sedma.Checked and SetFlek.Sedma.Enabled)
			or (SetFlek.Kilo.Checked and SetFlek.Kilo.Enabled) or PovinneKilo;
	end;
end;

function TPlayer.PossibleCards(Possible: TCards): byte;
var i: byte;
	Maybe: TCards;
begin
	if Trick.Count = 0 then begin
		Possible.Count:= Hand.Count;
		for i:= 1 to Hand.Count do Possible.Card[i]:= Hand.Card[i];
		Result:= 0;
	end
	else begin
		Maybe:= TCards.Create;
		Possible.Count:= 0;
		Result:= 1;

		for i:= 1 to Hand.Count do if Trick.Color = Hand.Card[i] div 8 then {Cti barvu!}
			Maybe.Add(Hand.Card[i]);
		if Maybe.Count = 0 then begin {Ty nem barvu?}
			Result:= 2;
			for i:= 1 to Hand.Count do if Trump = Hand.Card[i] div 8 then {Tak vynes trumfy!}
				Maybe.Add(Hand.Card[i]);
			if Maybe.Count = 0 then begin {Ty nem ani trumfy?}
				for i:= 1 to Hand.Count do {Tak vynes cokoliv...}
					Possible.Add(Hand.Card[i]);
				Result:= 3;
			end
			else {Pebij trumfa!}
				if Trick.Trump then {Trumfy u ly...}
					for i:= 1 to Maybe.Count do if Main.BetterCard(Maybe.Card[i], Trick.Best) then {Pebjej!}
						Possible.Add(Maybe.Card[i]);
		end
		else {M barvu}
			if not Trick.Trump or (Trick.Color = Trump) then {Trumfy jet nely nebo jsou na tychu...}
				for i:= 1 to Maybe.Count do if Main.BetterCard(Maybe.Card[i], Trick.Best) then {Pebjej!}
					Possible.Add(Maybe.Card[i]);

		if Possible.Count = 0 then for i:= 1 to Maybe.Count do Possible.Add(Maybe.Card[i]);
		Maybe.Free;
	end;
end;

function TPlayer.PossiblePlay(ACard: TCard): boolean;
var
	Possible: TCards;
begin
	Result:= False;
	Possible:= TCards.Create;
	PossibleCards(Possible);
	if Possible.IsThere(ACard) then Result:= True;
	Possible.Free;
end;

procedure TPlayer.Play;
var i, j: byte;
	Best: byte;
	Possible: TCards;
begin
	Possible:= TCards.Create;
	with Possible do begin
		for i:= 1 to 8 do Tag[i]:= 0;
		case PossibleCards(Possible) of
			0: {zakldm tych}
				for i:= 0 to 3 do begin {pro kadou barvu}
					if IsThere(8*i + 7) then {nejdv vynes eso}
						if (Stat[i].Zbyva - ColorCount(i) >= 3 + Ord(not Options.Risk.Checked)) then
							Inc(Tag[WhereIs(8*i + 7)], 100 + 5 * (Stat[i].Zbyva - ColorCount(i)))
						else if Stat[i].VzalTrumf then Dec(Tag[WhereIs(8*i + 7)], 100);
					if IsThere(8*i + 3) then {potom zkus destku}
						if Options.Risk.Checked and (ColorCount(i) = 1) and (Stat[i].Zbyva = 4) then
							Inc(Tag[WhereIs(8*i + 3)], 50)
						else Dec(Tag[WhereIs(8*i + 3)], 50);
					if Stat[i].VzalTrumf then {potom vytskej trumfy}
						for j:= 1 to Count do
							if Card[j] div 8 = i then Inc(Tag[j], 30);
				end;
			1: begin {ctm barvu}
				if IsThere(8*Trick.Color + 7) then {eso}
					if not Stat[Trick.Color].VzalTrumf and (Stat[Trick.Color].Zbyva + Trick.Count - ColorCount(Trick.Color)
					>= 3 + Ord(not Options.Risk.Checked and (Trick.Count <> 3)))
						then Inc(Tag[WhereIs(8*Trick.Color + 7)], 100)
					else Dec(Tag[WhereIs(8*Trick.Color + 7)], 100);
				if IsThere(8*Trick.Color + 3) then {destka}
					if (Enemies[Trick.Owner] = 1) and not Stat[Trick.Color].VzalTrumf and Stat[Trick.Color].Eso
					then Inc(Tag[WhereIs(8*Trick.Color + 3)], 50)
					else Dec(Tag[WhereIs(8*Trick.Color + 3)], 50);
			end;
			2: begin {trumfuju}
			end;
			3: begin {nesu cokoliv}
			end;
		end;
		for i:= 1 to Count do Inc(Tag[i], 8 - Card[i] mod 8); {vynej od nejni}
		for i:= 1 to Count do if Card[i] div 8 = Trump then Dec(Tag[i], 200); {nenes trumfy}
		Best:= 1;
		for i:= 2 to Count do if Tag[i] > Tag[Best] then Best:= i;
		Main.PlayCard(Card[Best]);
		Free;
	end;
end;

procedure THumanPlayer.Play;
var i: byte;
begin
	if not Options.Playing.Checked then inherited Play else begin
		Main.Hint:= 'Vyn';
		for i:= 1 to Hand.Count do CardImage[Hand.Card[i]].OnClick:= PlayClick;
	end;
end;

procedure THumanPlayer.PlayClick(Sender: TObject);
var i: byte;
	TheCard: TCard;
	HaveColor: boolean;
	S: string;
begin
	TheCard:= TCardImage(Sender).Tag;
	if (Trick.Count = 0) or PossiblePlay(TheCard) then begin
		Main.Hint:= '';
		for i:= 1 to Hand.Count do CardImage[Hand.Card[i]].OnClick:= nil;
		Main.PlayCard(TheCard);
	end
	else begin
		if TheCard div 8 = Trick.Color then S:= 'Mus pebjet'
		else begin
			HaveColor:= False;
			for i:= 1 to Hand.Count do if Hand.Card[i] div 8 = Trick.Color then HaveColor:= True;
			if HaveColor then S:= 'Mus ctt barvu'
			else if TheCard div 8 = Trump then
				S:= 'Mus pebt trumfy ve tychu'
			else S:= 'Mus vynst trumfy'
		end;
		if Options.ShowMistakes.Checked then MsgDlg(S, mtWarning, mb_OK)
		else begin
			CardImage[TheCard].Back;
			Main.ShowTimer.Enabled:= True;
			Main.ShowTimer.Tag:= TheCard;
			if Options.SoundMistakes.Checked then Beep;
			Main.Hint:= S;
		end;
	end;
end;

procedure TMain.PlayCard(ACard: TCard);
var i, j: byte;
begin
	Trick.Add(ACard);
	Player[Playing].Hand.Remove(ACard);

	if ACard = TrumpWith then for i:= 0 to 3 do
		if (i = Playing) or (i = Trumper) then for j:= 0 to 3 do
			if (j = Playing) or (j = Trumper) then
				Player[i].Enemies[j]:= 1
			else Player[i].Enemies[j]:= -1
		else for j:= 0 to 3 do
			if (j = Playing) or (j = Trumper) then
				Player[i].Enemies[j]:= -1
			else Player[i].Enemies[j]:= 1;

	CardImage[ACard].Front;
	CardImage[ACard].BringToFront;
	Player[Playing].ShowTrickCard(Trick.Count);
	Player[Playing].ShowCards;

	if (ACard mod 8 = 5) or (ACard mod 8 = 6) then with Player[Playing] do
		for i:= 1 to Hand.Count do if Hand.Card[i] = ACard + 1 - 2*(Ord(ACard mod 8 = 6)) then begin
			Inc(Calls, 20);
			if ACard div 8 = Trump then begin
				Player[Playing].Say('Trumfov hlka');
				Inc(Calls, 20);
				TrumpCall:= True;
			end
			else Player[Playing].Say('Hlka');
		end;

	if Trick.Count = 1 then begin
		Trick.Color:= ACard div 8;
		Trick.Best:= ACard;
		Trick.Owner:= Playing;
		Trick.Trump:= (ACard div 8 = Trump);
	end
	else begin
		Trick.Trump:= Trick.Trump or (ACard div 8 = Trump);
		if BetterCard(ACard, Trick.Best) then begin
			Trick.Best:= ACard;
			Trick.Owner:= Playing;
		end;
	end;

	with Stat[ACard div 8] do begin
		Dec(Zbyva);
		if ACard div 8 <> Trick.Card[1] div 8 then begin
			Prosla:= False;
			VzalTrumf:= ACard div 8 = Trump;
			VzalTrumfem:= Trick.Owner;
			end;
		if ACard mod 8 = 7 then Eso:= True;
	end;

	Playing:= (Playing + 1) mod 4;
	if Trick.Count = 4 then PrepareGetTrick
	else Player[Playing].Play;
end;

procedure TMain.GetTrick(Sender: TObject);
var i: byte;
begin
	GetTrickReset;
	for i:= 1 to 4 do
		if (Trick.Card[i] mod 8 = 7) or (Trick.Card[i] mod 8 = 3) then
			Inc(Player[Trick.Owner].Points, 10);
	for i:= 1 to 4 do CardImage[Trick.Card[i]].Hide;
	Refresh;
	if Player[0].Hand.Count = 0 then EndGame
	else begin
		Trick.Count:= 0;
		Playing:= Trick.Owner;
		Player[Playing].Play;
	end;
end;

procedure TMain.EndGame;
var i: byte;
	Human: TPoints;
	Enemy: TPoints;
	KiloBody: integer;
	KiloKrat: integer;
begin
	EndOfGame:= True;
	GamePanel.Hide;
	Human:= TPoints.Create;
	Enemy:= TPoints.Create;
	Inc(Player[Trick.Owner].Points, 10);
	for i:= 0 to 3 do
		if Player[0].Enemies[i] > 0 then Human.Add(i)
		else Enemy.Add(i);

	{Body za hru}
	for i:= 0 to 3 do TLabel(Results.FindComponent(Format('Points%d', [i]))).Caption:=
		Format('%d+%d = %d', [Player[i].Points, Player[i].Calls, Player[i].Points + Player[i].Calls]);
	if Human.Sum > Enemy.Sum then ResultColor:= clBlue else ResultColor:= clRed;

	{Hlen kilo}
	if Kilo <> 0 then begin
		if (Kilo < 0) xor Player[0].MeWith then with Human do
			if Hundred then KiloBody:= Sum
			else KiloBody:= Points + BestCall
		else with Enemy do
			if Hundred then KiloBody:= Sum
			else KiloBody:= Points + BestCall;
		if Kilo > 0 then Results.Hra.Caption:= Format('Kilo (%d)', [KiloBody])
		else Results.Hra.Caption:= Format('Kilo proti (%d)', [KiloBody]);
		if Abs(Kilo) > 1 then Results.Hra.Caption:= Results.Hra.Caption + ', ' + FlekText[Abs(Kilo) - 1];
		if KiloBody >= 100 then KiloKrat:= (KiloBody - 90) div 10 else KiloKrat:= (KiloBody - 100) div 10;
		for i:= 0 to 3 do Player[i].HraMoney:= -4 * PowerInt(2, Abs(Kilo) - 1) * KiloKrat
			* Ord11(Player[Trumper].MeWith xor Player[i].MeWith);
	end
	{Tich kilo}
	else if (Human.Sum >= 100) or (Enemy.Sum >= 100) then begin
		if Human.Sum > Enemy.Sum then KiloBody:= Human.Sum else KiloBody:= Enemy.Sum;
		Results.Hra.Caption:= Format('Tich kilo (%d)', [KiloBody]);
		if Flek > 0 then Results.Hra.Caption:= Results.Hra.Caption + ', ' + FlekText[Flek];
		for i:= 0 to 3 do Player[i].HraMoney:= - 2 * PowerInt(2, Flek) * (KiloBody - 90) div 10
			* Ord11((Player[0].Enemies[i] = 1) xor (Human.Sum = KiloBody));
	end
	{Normln hra}
	else begin
		if Flek > 0 then Results.Hra.Caption:= 'Hra, ' + FlekText[Flek]
		else Results.Hra.Caption:= 'Hra';
		for i:= 0 to 3 do Player[i].HraMoney:= PowerInt(2, Flek)
			* Player[0].Enemies[i] * Ord11(Human.Sum > Enemy.Sum);
	end;

	{Hlen sedma}
	if Sedma <> 0 then begin
		if (Trick.Card[(4 + Trick.Owner - Playing) mod 4 + 1] = 8*Trump)
		and (Player[Trumper].Enemies[Trick.Owner] = Sign(Sedma)) then begin {Uhran}
			for i:= 0 to 3 do Player[i].SedmaMoney:= -2 * PowerInt(2, Abs(Sedma) - 1) * Ord11(Player[i].MeWith xor (Sedma > 0));
			Results.Sedma.Caption:= '';
			end
		else begin {Neuhran}
			for i:= 0 to 3 do Player[i].SedmaMoney:= 2 * PowerInt(2, Abs(Sedma) - 1) * Ord11(Player[i].MeWith xor (Sedma > 0));
			Results.Sedma.Caption:= 'Neuhran ';
		end;
		if Sedma > 0 then Results.Sedma.Caption:= Capitalize(Results.Sedma.Caption + 'sedma')
		else Results.Sedma.Caption:= Capitalize(Results.Sedma.Caption + 'sedma proti');
		if Abs(Sedma) > 1 then Results.Sedma.Caption:= Results.Sedma.Caption + ', ' + FlekText[Abs(Sedma) - 1];
	end
	{Tich sedma}
	else if Trick.IsThere(8*Trump) then begin
		for i:= 0 to 3 do Player[i].SedmaMoney:=
			-Ord11(Player[(Trick.WhereIs(8*Trump) + 5 - Playing) mod 4].MeWith xor Player[i].MeWith)
			* Ord11(Trick.Card[(4 + Trick.Owner - Playing) mod 4 + 1] = 8*Trump);
		if Trick.Card[(4 + Trick.Owner - Playing) mod 4 + 1] = 8*Trump then Results.Sedma.Caption:= 'Tich sedma'
		else Results.Sedma.Caption:= 'Zabit sedma';
	end
	{dn sedma}
	else for i:= 0 to 3 do Player[i].SedmaMoney:= 0;

	Human.Free;
	Enemy.Free;
	ShowResults;
end;

procedure TMain.EndNoFlek;
var i: byte;
begin
	Player[Trumper].ShowCards;
	EndOfGame:= True;
	Player[Trumper].Say('Nikdo to nefleknul');
	Results.Hra.Caption:= 'Nehrlo se';
	for i:= 0 to 3 do Player[0].Enemies[i]:= -Ord11(Player[i].MeWith xor Player[0].MeWith);
	for i:= 0 to 3 do Player[Trumper].Enemies[i]:= Ord11(Player[i].Hand.IsThere(TrumpWith) or (i=Trumper));
	for i:= 0 to 3 do TLabel(Results.FindComponent(Format('Points%d', [i]))).Caption:= '0';
	if Player[0].Enemies[Trumper] = 1 then ResultColor:= clBlue else ResultColor:= clRed;
	for i:= 0 to 3 do Player[i].HraMoney:= Player[Trumper].Enemies[i];
	for i:= 0 to 3 do Player[i].SedmaMoney:= 0;
	ShowResults;
end;

procedure TMain.GameNewClick(Sender: TObject);
var i: byte;
begin
	if EndOfGame or not Options.Tournament.Checked then NewGame
	else if MsgDlg('Chcete zat novou hru a tm tuto ukonit renoncem?', mtConfirmation, mb_YesNo) = mrYes then begin
		for i:= 1 to Player[0].Hand.Count do begin
			CardImage[Player[0].Hand.Card [i]].OnClick:= nil;
			CardImage[Player[0].Hand.Card [i]].OnMouseDown:= nil;
		end;
		Player[0].Samec:= True;
		for i:= 1 to 3 do Player[i].Samec:= False;
		Results.Hra.Caption:= 'Renonc';
		for i:= 0 to 3 do Player[0].Enemies[i]:= Ord11(i=0);
		for i:= 0 to 3 do TLabel(Results.FindComponent(Format('Points%d', [i]))).Caption:= '0';
		ResultColor:= clRed;
		Player[0].HraMoney:= -10;
		for i:= 1 to 3 do Player[i].HraMoney:= 10;
		for i:= 0 to 3 do Player[i].SedmaMoney:= 0;
		ShowResults;
	end;
end;

procedure TMain.ShowResults;
var i: byte;
begin
	for i:= 0 to 3 do Player[i].HraMoney:= (1 + 2*Ord(Player[i].Samec)) * Player[i].HraMoney;
	Results.HraMoney.Caption:= Money(Player[0].HraMoney);
	if Player[0].HraMoney > 0 then Results.Hra.Font.Color:= clBlue else Results.Hra.Font.Color:= clRed;

	for i:= 0 to 3 do Player[i].SedmaMoney:= (1 + 2*Ord(Player[i].Samec)) * Player[i].SedmaMoney;
	Results.Sedma.Visible:= Player[0].SedmaMoney <> 0;
	Results.SedmaMoney.Visible:= Results.Sedma.Visible;
	Results.SedmaMoney.Caption:= Money(Player[0].SedmaMoney);
	if Player[0].SedmaMoney > 0 then Results.Sedma.Font.Color:= clBlue else Results.Sedma.Font.Color:= clRed;

	for i:= 0 to 3 do Player[i].HraMoney:= Player[i].HraMoney + Player[i].SedmaMoney;
	if (Trump <> 0) or (GameType <> 0) or not EndOfGame then Results.Celkem.Caption:= 'Celkem'
	else begin
		Results.Celkem.Caption:= 'Celkem, v lepch';
		for i:= 0 to 3 do Player[i].HraMoney:= 2 * Player[i].HraMoney;
	end;
	if Options.Tournament.Checked then for i:= 0 to 3 do Player[i].Money:= Player[i].Money + Player[i].HraMoney;
	Results.CelkemMoney.Caption:= Money(Player[0].HraMoney);
	if Player[0].HraMoney > 0 then Results.Celkem.Font.Color:= clBlue
	else if Player[0].HraMoney < 0 then Results.Celkem.Font.Color:= clRed
	else Results.Celkem.Font.Color:= clWindowText;
	Results.TogetherMoney.Caption:= Money(Player[0].Money);
	if Player[0].Money > 0 then Results.Together.Font.Color:= clBlue
	else if Player[0].Money < 0 then Results.Together.Font.Color:= clRed
	else Results.Together.Font.Color:= clWindowText;
	for i:= 0 to 3 do if i=Trumper then TLabel(Results.FindComponent(Format('Name%d', [i]))).Font.Style:= [fsBold]
	else TLabel(Results.FindComponent(Format('Name%d', [i]))).Font.Style:= [];
	for i:= 0 to 3 do
		if Player[0].Enemies[i] > 0 then TLabel(Results.FindComponent(Format('Name%d', [i]))).Font.Color:= ResultColor
		else TLabel(Results.FindComponent(Format('Name%d', [i]))).Font.Color:= clBlue - ResultColor + clRed;
	EndOfGame:= True;
	i:= ShowModalCenter(Results);
	if Results.Moneys.Checked then ShowModalCenter(Moneys);
	if i=mrOK then NewGame else if i=mrAbort then Close;
end;

{===========================================================================================}
{===========================================================================================}
{===========================================================================================}

procedure TMain.PrepareGetTrick;
var i: byte;
begin
	Main.OnClick:= GetTrickClick;
	SayPanel.OnClick:= GetTrickClick;
	SayLabel.OnClick:= GetTrickClick;
	for i:= 0 to 31 do CardImage[i].OnClick:= GetTrickClick;
	Main.OnKeyPress:= GetTrickKeyPress;
	TrickTimer.Enabled:= True;
	Player[Trick.Owner].SayGetTrick;
end;

procedure TMain.GetTrickKeyPress(Sender: TObject; var Key: Char);
begin
	GetTrick(Sender);
end;

procedure TMain.GetTrickClick(Sender: TObject);
begin
	GetTrick(Sender);
end;

procedure TMain.GetTrickReset;
var i: byte;
begin
	SayPanel.Visible:= False;
	TrickTimer.Enabled:= False;
	Main.OnClick:= nil;
	SayPanel.OnClick:= nil;
	SayLabel.OnClick:= nil;
	for i:= 0 to 31 do CardImage[i].OnClick:= nil;
	Main.OnKeyPress:= FormKeyPress;
end;

constructor TPlayer.Create;
begin
	inherited Create;
	Hand:= TCards.Create;
end;

function TMain.BetterRank(Better, Worse: TCard): boolean;
begin
	Result:= Better mod 8 > Worse mod 8;
	if GameType = 0 then begin
		if Better mod 8 = 3 then Result:= Worse mod 8 <> 7;
		if Worse mod 8 = 3 then Result:= Better mod 8 = 7;
	end;
end;

function TMain.BetterCard(Better, Worse: TCard): boolean;
begin
	Result:= (Better div 8 = Worse div 8) and BetterRank(Better, Worse);
	if GameType = 0 then Result:= (Result or ((Better div 8 = Trump) and (Worse div 8 <> Trump)));
end;

procedure TMain.MixCards;
var i: byte;
	A: array[TCard] of boolean;
	RndCard: TCard;
begin
	for i:= 0 to 31 do A[i]:= False;
	Packet.Count:= 0;
	while (Packet.Count <> 32) do begin
		RndCard:= Random(32);
		while A[RndCard] do RndCard:= (RndCard + 1) mod 32;
		Packet.Add(RndCard);
		A[RndCard]:= True;
	end;
end;

procedure TMain.CutIt(Position: TCard);
var i: byte;
	Temp: TCards;
begin
	Temp:= TCards.Create;
	for i:= 1 to Position do Temp.Card[i]:= Packet.Card[32 - Position + i];
	for i:= Position + 1 to 32 do Temp.Card[i]:= Packet.Card[i - Position];
	Packet.Card:= Temp.Card;
	Temp.Free;
end;

procedure TPlayer.CutCards;
begin
	Main.CutIt(Random(30) + 2);
end;

procedure THumanPlayer.CutCards;
begin
	if not Options.Cutting.Checked then inherited CutCards
	else begin
		SetCut.CutBar.Position:= Random(30) + 2;
		Main.ShowModalCenter(SetCut);
		Main.CutIt(SetCut.CutBar.Position);
	end;
end;

procedure TPlayer.Say(S: string);
begin
	with SayIt do begin
		SayLabel.Caption:= S;
		ClientWidth:= SayLabel.Width + 2*SayLabel.Left;
		ClientHeight:= SayLabel.Height + 2*SayLabel.Top;
		Left:= Main.Left + Main.Width - Main.ClientWidth + (Main.ClientWidth - Width) div 2;
		Top:= Main.Top + Main.Height - Main.ClientHeight + (Main.ClientHeight - Main.StatusBar.Height - Height) div 2;
		case Me of
			0: Top:= Top + 2*CardSpace.Y;
			1: Left:= Left - 4*CardSpace.X;
			2: Top:= Top - 2*CardSpace.Y;
			3: Left:= Left + 4*CardSpace.X;
		end;
		ShowModal;
	end;
end;

procedure TPlayer.SayGetTrick;
begin
	with Main.SayPanel do begin
		Left:= Main.Width - Main.ClientWidth + (Main.ClientWidth - Width) div 2;
		Top:= (Main.ClientHeight - Main.StatusBar.Height - Height) div 2;
		case Me of
			0: Top:= Top + 2*CardSpace.Y;
			1: Left:= Left - 4*CardSpace.X;
			2: Top:= Top - 2*CardSpace.Y;
			3: Left:= Left + 4*CardSpace.X;
		end;
		Visible:= True;
	end;
end;

procedure TCards.Add(ACard: TCard);
begin
	Inc(Count);
	Card[Count]:= ACard;
end;

procedure TCards.Remove(ACard: TCard);
var i, j: byte;
begin
	for i:= 1 to Count do if Card[i] = ACard then begin
		for j:= i + 1 to Count do Card[j - 1]:= Card[j];
		Dec(Count);
	end;
end;

procedure TCards.Sort;
var i, j: byte;
	Temp: array[TCard] of boolean;
begin
	for i:= 0 to 31 do Temp[i]:= False;
	for i:= 1 to Count do Temp[Card[i]]:= True;
	j:= 1;
	for i:= 0 to 31 do if Temp[i] then begin
		Card[j]:= i;
		Inc(j);
	end;
end;

procedure TMain.MoveControl(Control: TControl; X, Y: integer);
begin
	Control.Left:= X;
	Control.Top:= Y;
end;

procedure TMain.MoveCard(ACard: TCard; X, Y: integer);
begin
	MoveControl(CardImage[ACard], X, Y);
end;

procedure TMain.GameExitClick(Sender: TObject);
begin
	Close;
end;

function TCards.WhereIs(ACard: TCard): byte;
var i: byte;
begin
	Result:= 0;
	for i:= 1 to Count do if Card[i] = ACard then Result:= i;
end;

function TCards.IsThere(ACard: TCard): boolean;
begin
	Result:= WhereIs(ACard) <> 0;
end;

procedure TMain.FormHint(Sender: TObject);
begin
	StatusBar.SimpleText:= Application.Hint;
end;

procedure TPoints.Add(APlayer: TPlayers);
begin
	Inc(Points, Player[APlayer].Points);
	Inc(Calls, Player[APlayer].Calls);
	TrumpCall:= TrumpCall or Player[APlayer].TrumpCall;
end;

function TPoints.Hundred: boolean;
begin
	Result:= ((Points >= 60) and TrumpCall) or ((Points >= 80) and (Calls >= 20))
end;

function TPoints.Sum: byte;
begin
	Result:= Points + Calls;
end;

function TPoints.BestCall: byte;
begin
	if TrumpCall then Result:= 40 else Result:= Min(20, Calls);
end;

function TMain.ShowModalCenter(AForm: TForm): integer;
var
	OldHint: string;
begin
	OldHint:= Main.Hint;
	Main.Hint:= '';
	AForm.Left:= Main.Left + (Main.Width - AForm.Width) div 2;
	AForm.Top:= Main.Top + (Main.Height - AForm.Height) div 2;
	AForm.ActiveControl:= nil;
	Result:= AForm.ShowModal;
	Main.Hint:= OldHint;
end;

function TCards.ColorCount(AColor: TCardColor): byte;
var i: byte;
begin
	Result:= 0;
	for i:= 1 to Count do if Card[i] div 8 = AColor then Inc(Result);
end;

function TCards.BestCards(AColor: TCardColor): byte;
var i: byte;
begin
	i:= 7;
	Result:= 0;
	while IsThere(8*AColor + i) do begin
		if (GameType = 0) then
			if i=7 then Dec(i, 4)
			else if i=3 then Inc(i, 3)
			else Dec(i)
		else Inc(i);
		Inc(Result);
	end;
end;

procedure TMain.GameOptionsClick(Sender: TObject);
begin
	if ShowModalCenter(Options) <> mrOk then Options.Load
	else begin
		Options.Save;
		Options.Apply;
	end;
end;

procedure TMain.HelpAboutClick(Sender: TObject);
begin
	ShowModalCenter(About);
end;

procedure TMain.HelpTopicsClick(Sender: TObject);
begin
	Application.HelpCommand(HELP_FINDER, 0);
end;

procedure TMain.GameMoneyClick(Sender: TObject);
begin
	ShowModalCenter(Moneys);
end;

procedure TMain.HelpTipsClick(Sender: TObject);
begin
	ShowModalCenter(Tips);
end;

procedure TMain.FormKeyPress(Sender: TObject; var Key: Char);
begin
	case Key of
		#27: Application.Minimize;
	end;
end;

procedure TCardImage.Front;
begin
	Show;
	Picture.Assign(CardFront[Tag]);
	IsFront:= True;
end;

procedure TCardImage.Back;
begin
	Show;
	Picture.Assign(CardBack);
	IsFront:= False;
end;

procedure TPlayer.ShowCards;
var i, j: byte;
	X, Y: integer;
	Lines: byte;
	OddLines: byte;
	Showing: TCards;
begin
	if Hand.Count > 0 then begin
		Showing:= TCards.Create;
		Showing.Count:= 0;
		for i:= 1 to Hand.Count do if CardImage[Hand.Card[i]].IsFront then Showing.Add(Hand.Card[i]);
		for i:= 1 to Hand.Count do if not CardImage[Hand.Card[i]].IsFront then Showing.Add(Hand.Card[i]);
		Lines:= Ord(Odd(Me) and (Showing.Count > 3)) + 1;
		OddLines:= Ord(Odd(Showing.Count) and (Lines > 1));

		if Odd(Me) then Y:= (Main.ClientHeight - Main.StatusBar.Height - CardSize.Y - Ord(Lines=2) * CardSpace.Y) div 2
		else X:= Main.ClientWidth div 2 - ((Showing.Count-1) * CardSpace.X + CardSize.X) div 2;
		if Me=0 then Y:= (Main.ClientHeight - Main.StatusBar.Height) - 12 - CardSize.Y
		else if Me=2 then Y:= 12;

		for j:= 1 to Lines do begin
			if Me=1 then X:= 12 + Ord(j=1) * OddLines * CardSpace.X div 2 + Ord((Lines=2) and (j=1) and not Odd(Showing.Count)) * 16
			else if Me=3 then X:= Main.ClientWidth - 12 - CardSpace.X * (Showing.Count div Lines - 1) - CardSize.X
				- Ord(Odd(Showing.Count) and (j=1) and (Lines=2)) * CardSpace.X div 2
				- Ord(j=2) * OddLines * CardSpace.X - Ord((j=2) and not Odd(Showing.Count)) * 12;
			for i:= 1 to Showing.Count div Lines + OddLines * Ord(j=2) do begin
				Main.MoveCard(Showing.Card[(j-1) * Showing.Count div Lines + i], X, Y);
				Inc(X, CardSpace.X);
			end;
			Inc(Y, CardSpace.Y);
		end;
		Showing.Free;
	end;
end;

procedure TPlayer.ShowTrickCard(TrickCard: byte);
var
	X, Y: integer;
begin
	X:= (Main.ClientWidth - CardSize.X) div 2;
	Y:= (Main.ClientHeight - Main.StatusBar.Height - CardSize.Y) div 2;
	case Me of
		0: begin Inc(X, 5); Inc(Y, CardSpace.Y div 2); end;
		1: begin Dec(X, CardSpace.X); Inc(Y, 10); end;
		2: begin Dec(X, 5); Dec(Y, CardSpace.Y div 2); end;
		3: begin Inc(X, CardSpace.X); Dec(Y, 10); end;
	end;
	Main.MoveCard(Trick.Card[TrickCard], X, Y);
end;

procedure TMain.ShowGameInfo(APlayer: TPlayers);
begin
	case APlayer of
		0: MoveControl(GamePanel, 45, ClientHeight - StatusBar.Height - GamePanel.Height - 45);
		1: MoveControl(GamePanel, 12, 90);
		2: MoveControl(GamePanel, ClientWidth - GamePanel.Width - 45, 45);
		3: MoveControl(GamePanel, ClientWidth - GamePanel.Width - 12, ClientHeight - StatusBar.Height - GamePanel.Height - 90);
	end;
end;

procedure TMain.ShowLines;
var
	LineSpace: integer;
begin
	LineSpace:= 128;
	HLine1.Width:= ClientWidth - LineSpace + 20;
	HLine2.Width:= ClientWidth - LineSpace;
	HLine2.Top:= ClientHeight - StatusBar.Height - LineSpace;
	VLine2.Left:= 40 + 3*CardSpace.X + CardSize.X;
	VLine2.Height:= ClientHeight - StatusBar.Height - 2*LineSpace;
	VLine3.Height:= ClientHeight - StatusBar.Height - 2*LineSpace;
	VLine3.Left:= ClientWidth - 40 - 3*CardSpace.X - CardSize.X;
	VLine4.Top:= ClientHeight - StatusBar.Height - LineSpace + 1;
	VLine4.Left:= ClientWidth - LineSpace;
end;

procedure TMain.FormResize(Sender: TObject);
var i: byte;
begin
	if not Application.Terminated then begin
		ShowGameInfo(Trumper);
		CardSpace.X:= ClientWidth div 14;
		CardSpace.Y:= Sqr(ClientHeight div 60);
		for i:= 1 to Trick.Count do Player[(3 + Playing-Trick.Count + i) mod 4].ShowTrickCard(i);
		for i:= 0 to 3 do Player[i].ShowCards;
		ShowLines;
	end;
end;

procedure TMain.FormClose(Sender: TObject; var Action: TCloseAction);
var i: byte;
begin
	if not EndOfGame and Options.Tournament.Checked then
		if MsgDlg('Chcete skonit a tm tuto hru ukonit renoncem?', mtConfirmation, mb_YesNo) = mrNo then Action:= caNone
		else begin
			Player[0].Money:= Player[0].Money - 30;
			for i:= 1 to 3 do Player[i].Money:= Player[i].Money + 10;
		end;
	Moneys.Save;
end;

procedure TMain.ShowTimerTimer(Sender: TObject);
begin
	ShowTimer.Enabled:= False;
	CardImage[ShowTimer.Tag].Front;
end;

end.

