unit _Options;

interface
uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, ComCtrls, Utils, ExtCtrls;

type
	TOptions = class(TForm)
    OK: TButton;
    Cancel: TButton;
		Preferences: TPageControl;
    Settings: TTabSheet;
    Names: TTabSheet;
    Automatic: TTabSheet;
    Label1: TLabel;
    Cutting: TCheckBox;
    Trumping: TCheckBox;
    Coloring: TCheckBox;
    Fleking: TCheckBox;
    Playing: TCheckBox;
    Name0: TEdit;
    Name1: TEdit;
    Name2: TEdit;
    Name3: TEdit;
    Appearance: TTabSheet;
    StatusLine: TCheckBox;
		GuideLines: TCheckBox;
		SpeechSpeedLabel: TLabel;
		TrickSpeedLabel: TLabel;
		SpeechSpeed: TEdit;
		SpeechSpeedUpDown: TUpDown;
		TrickSpeed: TEdit;
		TrickSpeedUpDown: TUpDown;
		Label2: TLabel;
		Risk: TCheckBox;
		StartNewGame: TCheckBox;
		Tournament: TCheckBox;
		Mixing: TCheckBox;
    ShowMistakes: TCheckBox;
    SoundMistakes: TCheckBox;
    OpenGame: TCheckBox;
		procedure Load;
		procedure Save;
		procedure Apply;
		procedure UpDownEditKeyPress(Sender: TObject; var Key: Char);
		procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
		procedure TournamentClick(Sender: TObject);
		procedure FormShow(Sender: TObject);
    procedure ShowMistakesClick(Sender: TObject);
    procedure UpDownEditKeyPress2(Sender: TObject; var Key: Char);
	end;

var Options: TOptions;

implementation
{$R *.DFM}
uses _Main, _SayIt, _Results, _Moneys;

var
	OldTournament: boolean;

procedure TOptions.UpDownEditKeyPress(Sender: TObject; var Key: Char);
begin
	if not (Key in ['0'..'9', Chr(vk_Back)]) then begin
		Key:= #0;
		Beep;
	end;
end;

procedure TOptions.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	if (mbRight = Button) and (TWinControl(Sender).HelpContext <> 0) then
		Application.HelpCommand(HELP_CONTEXTPOPUP, TWinControl(Sender).HelpContext);
end;

procedure TOptions.FormShow(Sender: TObject);
begin
	OldTournament:= Tournament.Checked;
	TournamentClick(Sender);
	ShowMistakesClick(Sender);
	Preferences.ActivePage:= Settings;
end;

procedure TOptions.TournamentClick(Sender: TObject);
begin
	OpenGame.Enabled:= not Tournament.Checked;
	if Tournament.Checked then OpenGame.Checked:= False;
end;

procedure TOptions.ShowMistakesClick(Sender: TObject);
begin
	SoundMistakes.Enabled:= not ShowMistakes.Checked;
	if ShowMistakes.Checked then SoundMistakes.Checked:= True;
end;

procedure TOptions.Load;
const DefaultName: array [0..3] of string = ('Jakub', 'Honza', 'Petr', 'Jarda');
var i: byte;
begin
	with TRegIni.Create('Software\Jakub Vrna\tyka') do try
		Tournament.Checked:= ReadBool('Tournament', True);
		StartNewGame.Checked:= ReadBool('StartNewGame', True);
		Mixing.Checked:= ReadBool('Mixing', True);
		Risk.Checked:= ReadBool('Risk', True);
		ShowMistakes.Checked:= ReadBool('ShowMistakes', True);
		SoundMistakes.Checked:= ReadBool('SoundMistakes', True);
		for i:= 0 to 3 do TEdit(FindComponent(Format('Name%d', [i]))).Text:= ReadString(Format('Name%d', [i]), DefaultName [i]);
		Cutting.Checked:= ReadBool('Cutting', False);
		Trumping.Checked:= ReadBool('Trumping', True);
		Coloring.Checked:= ReadBool('Coloring', True);
		Fleking.Checked:= ReadBool('Fleking', True);
		Playing.Checked:= ReadBool('Playing', True);
		OpenGame.Checked:= ReadBool('OpenGame', False);
		StatusLine.Checked:= ReadBool('StatusLine', True);
		GuideLines.Checked:= ReadBool('GuideLines', False);
		SpeechSpeed.Text:= IntToStr(ReadInteger('SpeechSpeed', 10));
		TrickSpeed.Text:= IntToStr(ReadInteger('TrickSpeed', 0));
	finally Free;
	end;
end;

procedure TOptions.Save;
var i: byte;
begin
	with TRegIni.Create('Software\Jakub Vrna\tyka') do try
		WriteBool('Tournament', Tournament.Checked);
		WriteBool('StartNewGame', StartNewGame.Checked);
		WriteBool('Mixing', Mixing.Checked);
		WriteBool('Risk', Risk.Checked);
		WriteBool('ShowMistakes', ShowMistakes.Checked);
		WriteBool('SoundMistakes', SoundMistakes.Checked);
		for i:= 0 to 3 do WriteString(Format('Name%d', [i]), TEdit(FindComponent(Format('Name%d', [i]))).Text);
		WriteBool('Cutting', Cutting.Checked);
		WriteBool('Trumping', Trumping.Checked);
		WriteBool('Coloring', Coloring.Checked);
		WriteBool('Fleking', Fleking.Checked);
		WriteBool('Playing', Playing.Checked);
		WriteBool('OpenGame', OpenGame.Checked);
		WriteBool('StatusLine', StatusLine.Checked);
		WriteBool('GuideLines', GuideLines.Checked);
		WriteInteger('SpeechSpeed', Value(SpeechSpeed));
		WriteInteger('TrickSpeed', Value(TrickSpeed));
	finally Free;
	end;
end;

procedure TOptions.Apply;
var i, j: byte;
begin
	if not Tournament.Checked and OldTournament and
		(MsgDlg('Chcete ukonit turnaj a tm nastavit vchoz obnosy penz?', mtConfirmation, mb_YesNo) = mrNo)
	then Tournament.Checked:= True;
	if Tournament.Checked then Moneys.Info.Hide else Moneys.Info.Show;
	Results.Together.Visible:= Tournament.Checked;
	if not Tournament.Checked then for i:= 0 to 3 do Player [i].Money:= 0;
	Results.TogetherMoney.Visible:= Tournament.Checked;
	if not Tournament.Checked then Results.Moneys.Checked:= False;
	for i:= 0 to 3 do TLabel(Results.FindComponent(Format('Name%d', [i]))).Caption:=
		TEdit(FindComponent(Format('Name%d', [i]))).Text;
	for i:= 0 to 3 do TLabel(Moneys.FindComponent(Format('Name%d', [i]))).Caption:=
		TEdit(FindComponent(Format('Name%d', [i]))).Text;
	for i:= 1 to 3 do with Main do begin
		if OpenGame.Checked then for j:= 1 to Player [i].Hand.Count do CardImage [Player [i].Hand.Card [j]].Front
		else for j:= 1 to Player [i].Hand.Count do CardImage [Player [i].Hand.Card [j]].Back;
		Player [i].ShowCards;
	end;
	Main.StatusBar.Height:= 19 * Ord(StatusLine.Checked);
	for i:= 1 to 2 do TBevel(Main.FindComponent(Format('HLine%d', [i]))).Visible:= GuideLines.Checked;
	for i:= 1 to 4 do TBevel(Main.FindComponent(Format('VLine%d', [i]))).Visible:= GuideLines.Checked;
	SayIt.Timer.Interval:= 100 * Value(SpeechSpeed);
	Main.TrickTimer.Interval:= 100 * Value(TrickSpeed);
	Main.FormResize(Self);
end;

procedure TOptions.UpDownEditKeyPress2(Sender: TObject; var Key: Char);
begin
	if not (Key in ['0'..'9', Chr(vk_Back)]) then begin
		Key:= #0;
		Beep;
	end;
end;

end.

